package laso2016;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.StringTokenizer;

public class Main
{
	public static void main(String[] args) throws IOException
	{
		// this is the source file we are about to read from
		BufferedReader br = Files.newBufferedReader(Paths.get("graphweighted.txt"));

		// read number of edges and vertices
		StringTokenizer st = new StringTokenizer(br.readLine());
		final int N = Integer.parseInt(st.nextToken());
		final int M = Integer.parseInt(st.nextToken());

		// create the adjacency matrix, fill it with -1 denoting that edges do not exist
		final int[][] adj = new int[N][N];
		for (int i = 0; i < N; i++)
			Arrays.fill(adj[i], -1);

		// and read all the edges in the graph
		for (int i = 0; i < M; i++)
		{
			st = new StringTokenizer(br.readLine());
			final int u = Integer.parseInt(st.nextToken());
			final int v = Integer.parseInt(st.nextToken());
			final int weight = Integer.parseInt(st.nextToken());
			adj[u][v] = adj[v][u] = weight;
		}
		
		int[] sp = dijkstra(adj, 0);
		for(int i : sp)
			System.out.println(Integer.toString(i));
	}
	
	public static int[] dijkstra(int[][] adj, int s) {
		// initialize
		// array A will contain the length of the shortest path
		final int[] A = new int[adj.length];
		Arrays.fill(A, Integer.MAX_VALUE);
		A[s] = 0; // which is known only for s
		// this is last step of the shortest path, the predecessor in the shortest path
		final int[] predecessors = new int[adj.length];
		Arrays.fill(predecessors, -1);
		// the Dijkstra greedy score for vertices that are candidates for expansion of X
		final int[] score = new int[adj.length];
		Arrays.fill(score, Integer.MAX_VALUE);
		score[s] = 0; // X does not contain s yet, just programming trick so that updateScore method works
		updateScore(adj, s, score, predecessors, A);
		
		while(true) {
			// find a candidate to add to X, it is the vertex with the lowest score
			final int minIndex = findMinIndex(score);
			if(score[minIndex] == Integer.MAX_VALUE)
				break; // actually there is nowhere to grow X
			
			// now we grow X by edge from src[minIndex] to minIndex
			A[minIndex] = score[minIndex];
			updateScore(adj, minIndex, score, predecessors, A);
		}
		
		return A;
	}
	
	// finds the vertex with the minimum score - i.e. the vertex which will be added to X
	private static int findMinIndex(int[] score) {
		int minIndex = 0; // just trivial search for minimum
		for(int i = 1; i < score.length; i++)
			if(score[i] < score[minIndex])
				minIndex = i;
		return minIndex;
	}
	
	// updates the scores when the vertex srcV is added to the set X
	private static void updateScore(int[][] adj, int srcV, int[] score, int[] predecessors, int[] A) {
		for(int i = 0; i < adj.length; i++) // look on each other vertex
			if(adj[srcV][i] != -1 && A[i] == Integer.MAX_VALUE) // check whether there is edge from srcV to i and that i is not in X
				if(score[i] > score[srcV] + adj[srcV][i]) { // if we can improve the score ...
					score[i] = score[srcV] + adj[srcV][i]; // ... let's improve it and ...
					predecessors[i] = srcV; // ... and remember that we can get to i from srcV
				}
		score[srcV] = Integer.MAX_VALUE; // we have to set score of srcV to infty (same as removing from the heap)
	}
}
